@@ -42,6 +42,16 @@ def arithmetic_error():
     )
 
 
+@register("AssertionError")
+def assertion_error():
+    _ = current_lang.translate
+    return _(
+        "In Python, the keyword `assert` is used in statements of the form\n"
+        "`assert condition`, to confirm that `condition` is not `False`.\n\n"
+        "If `condition` is `False`, an `AssertionError` is raised.\n"
+    )
+
+
 @register("AttributeError")
 def attribute_error():
     _ = current_lang.translate
@@ -163,6 +173,16 @@ def recursion_error():
     )
 
 
+@register("StopIteration")
+def stop_iteration():
+    _ = current_lang.translate
+    return _(
+        "`StopIteration` is raised to indicate that an iterator has no more\n"
+        "item to provide when its `__next__` method is called by\n"
+        "the `next()` builtin function.\n"
+    )
+
+
 @register("SyntaxError")
 def syntax_error():
     _ = current_lang.translate
