@@ -30,7 +30,8 @@ class LangState:
         self.translate = None
         self.lang = "en"
 
-    def no_translation(self, text):  # noqa
+    @staticmethod
+    def no_translation(text):
         return text
 
     def install(self, lang=None):
@@ -63,10 +64,7 @@ class LangState:
                 # is not available.
             )
         self.lang = lang
-        if lang == "en":
-            self.translate = self.no_translation
-        else:
-            self.translate = _lang.gettext
+        self.translate = self.no_translation if lang == "en" else _lang.gettext
 
 
 current_lang = LangState()  # noqa
@@ -77,7 +75,7 @@ def no_information():
     return _(
         "No information is known about this exception.\n"
         "Please report this example to\n"
-        "https://github.com/aroberge/friendly-traceback/issues\n\n"
+        "https://github.com/aroberge/friendly/issues\n\n"
         "If you are using the Friendly console, use `www()` to\n"
         "do an Internet search for this particular case.\n"
     )
@@ -88,7 +86,7 @@ def internal_error():
     return _(
         "Internal error for Friendly-traceback.\n"
         "Please report this example to\n"
-        "https://github.com/aroberge/friendly-traceback/issues\n"
+        "https://github.com/aroberge/friendly/issues\n"
     )
 
 
