@@ -7,6 +7,7 @@ import sys
 from . import fixers
 from ..my_gettext import current_lang, internal_error
 from .. import debug_helper
+from .. import utils
 
 STATEMENT_ANALYZERS = []
 
@@ -53,7 +54,7 @@ def analyze_def_statement(statement):
         if statement is None:
             return {}
 
-    if statement.tokens[1] == "=" or statement.tokens[1] == ":=":
+    if str(statement.tokens[1]) in ("=", ":="):
         # Let the generic method handle the wrong assignment case
         return {}
 
@@ -71,7 +72,7 @@ def remove_async(statement):
     def ...
     """
 
-    if not (statement.tokens[0] == "async" and statement.tokens[1] == "def"):
+    if statement.tokens[0] != "async" or statement.tokens[1] != "def":
         debug_helper.log("Problem in remove_async: inconsistent state")
         return None
 
@@ -100,13 +101,6 @@ def remove_async(statement):
     return statement
 
 
-# TODO: rewrite this so that we first look at a series of fixers
-# like adding a colon at the end, replacing the last token by a colon,
-# adding parens, replacing the function name, etc. As soon as we find
-# a good fix, we can stop.  Otherwise, we continue the analysis as we currently
-# have.
-
-
 @add_statement_analyzer
 def def_begin_code_block(statement):  #
     # Thinking of trying to use def to begin a code block, i.e.
@@ -116,15 +110,17 @@ def def_begin_code_block(statement):  #
         return {}
 
     if statement.first_token.start_col == 0:
+        hint = _("A function needs a name.\n")
         cause = _(
             "You tried to define a function and did not use the correct syntax.\n"
         )
     else:
+        hint = _("Functions and methods need a name.\n")
         cause = _(
             "You tried to define a function or method and did not use the correct syntax.\n"
         )
 
-    return {"cause": cause + def_correct_syntax()}
+    return {"cause": cause + def_correct_syntax(), "suggest": hint}
 
 
 @add_statement_analyzer
@@ -188,8 +184,6 @@ def missing_colon(statement):
     """look for missing colon at the end of statement; includes the case where
     something else has been written as a typo."""
     _ = current_lang.translate
-
-    # TODO: need to add test for each case
     if (
         statement.last_token == ":"
         or statement.bad_token != statement.last_token
@@ -214,7 +208,7 @@ def missing_colon(statement):
     if fixers.check_statement(new_statement):
         hint = _("Did you write something by mistake after the colon?\n")
         cause += _("A block of code must come after the colon.\n")
-        cause += _("If you remove `{bad}`, this will fix the problem.\n").format(
+        cause += _("Removing `{bad}`, might fix the problem.\n").format(
             bad=statement.bad_token
         )
         return {"cause": cause, "suggest": hint}
@@ -243,9 +237,9 @@ def not_enough_tokens(statement):
         )
         if not fixers.check_statement(new_statement):
             return {"cause": cause + def_correct_syntax()}
-        else:
-            cause = hint = _("You forgot to name your function.\n")
-            return {"cause": cause + def_correct_syntax(), "suggest": hint}
+
+        cause = hint = _("You forgot to name your function.\n")
+        return {"cause": cause + def_correct_syntax(), "suggest": hint}
 
     return {"cause": cause + def_correct_syntax()}
 
@@ -319,9 +313,9 @@ def function_definition_missing_name(statement):
     )
     if fixers.check_statement(new_statement):
         return {"cause": cause + def_correct_syntax()}
-    else:
-        cause += _("However, there are some other syntax errors in your code.\n")
-        return {"cause": cause + def_correct_syntax()}
+
+    cause += _("However, there are some other syntax errors in your code.\n")
+    return {"cause": cause + def_correct_syntax()}
 
 
 @add_statement_analyzer
@@ -362,9 +356,7 @@ def dotted_name_not_allowed(statement):
 def positional_arguments_in_def(statement):
     _ = current_lang.translate
 
-    # TODO: add tests
-
-    if not (statement.bad_token == "/" and statement.prev_token.is_in("(,")):
+    if statement.bad_token != "/" or statement.prev_token.string not in "(,":
         return {}
 
     meaning = _(
@@ -384,13 +376,14 @@ def positional_arguments_in_def(statement):
 
     prev_tok = ""
     for tok in statement.tokens[0 : statement.bad_token_index]:
-        if tok == "=" or tok == "**":
+        if tok == "**":
             cause = meaning + _(
-                "You have some keyword arguments that appear before\n"
+                "You have unspecified keyword arguments that appear before\n"
                 "the symbol `/`.\n"
             )
-            hint = _("Keyword arguments must appear after the `/` symbol.\n")
+            hint = _("Keyword arguments must appear after the `/` symbol.\n")  # yes
             return {"cause": cause, "suggest": hint}
+
         if prev_tok == "*":  # might be incorrect if used in *args
             if tok == ",":
                 cause = meaning + _(
@@ -405,26 +398,32 @@ def positional_arguments_in_def(statement):
                 ).format(name=tok.string)
                 cause = meaning + hint
             return {"cause": cause, "suggest": hint}
-        elif tok == "/" and prev_tok == ",":
+
+        if tok == "/" and prev_tok == ",":
             cause = _("You can only use `/` once in a function definition.\n")
             return {"cause": cause, "suggest": cause}
+
         prev_tok = tok
+
     return {}
 
 
 @add_statement_analyzer
 def keyword_arguments_in_def(statement):
     _ = current_lang.translate
-    # TODO: add tests
-
-    if not (statement.bad_token == "*" and statement.prev_token == ","):
+    if statement.bad_token != "*" or statement.prev_token != ",":
         return {}
 
     for tok in statement.tokens[0 : statement.bad_token_index]:
         if tok == "*":
-            cause = _("You can only use `*` once in a function definition.\n")
-            return {"cause": cause, "suggest": cause}
-        elif tok == "**" or tok == "=":
+            hint = _("You can only use `*` once in a function definition.\n")
+            cause = hint + _(
+                "It must either be used by itself, `..., *, ...`,\n"
+                "or in the form `..., *args ...`, but not both.\n"
+            )
+            return {"cause": cause, "suggest": hint}
+
+        if str(tok) in ("**", "="):
             if statement.next_token.is_identifier():
                 cause = _(
                     "`*{name}` must appear before any keyword argument.\n"
@@ -439,7 +438,7 @@ def keyword_arguments_in_def(statement):
 def number_as_argument(statement):
     _ = current_lang.translate
 
-    if not (statement.bad_token.is_number() and statement.prev_token.is_in("(,")):
+    if not (statement.bad_token.is_number() and statement.prev_token.string in "(,"):
         return {}
 
     hint = _("You cannot use numbers as function arguments.\n")
@@ -454,7 +453,7 @@ def number_as_argument(statement):
 def string_as_argument(statement):
     _ = current_lang.translate
 
-    if not (statement.bad_token.is_string() and statement.prev_token.is_in("(,")):
+    if not (statement.bad_token.is_string() and statement.prev_token.string in "(,"):
         return {}
 
     hint = _("You cannot use strings as function arguments.\n")
@@ -469,7 +468,7 @@ def string_as_argument(statement):
 def tuple_as_argument(statement):
     _ = current_lang.translate
 
-    if not (statement.bad_token == "(" and statement.prev_token.is_in("(,")):
+    if statement.bad_token != "(" or statement.prev_token.string not in "(,":
         return {}
 
     hint = _("You cannot have explicit tuples as function arguments.\n")
@@ -485,9 +484,7 @@ def tuple_as_argument(statement):
 @add_statement_analyzer
 def list_as_argument(statement):
     _ = current_lang.translate
-    # TODO: add tests
-
-    if not (statement.bad_token == "[(]" and statement.prev_token.is_in("(,")):
+    if statement.bad_token != "[" or statement.prev_token.string not in "(,":
         return {}
 
     hint = _("You cannot have explicit lists as function arguments.\n")
@@ -500,9 +497,7 @@ def list_as_argument(statement):
 @add_statement_analyzer
 def dict_or_set_as_argument(statement):
     _ = current_lang.translate
-    # TODO: add tests
-
-    if not (statement.bad_token == "[{]" and statement.prev_token.is_in("(,")):
+    if statement.bad_token != "{" or statement.prev_token.string not in "(,":
         return {}
 
     hint = _("You cannot have any explicit dict or set as function arguments.\n")
@@ -514,46 +509,108 @@ def dict_or_set_as_argument(statement):
 
 @add_statement_analyzer
 def operator_as_argument(statement):
+    """This looks at various possible fixes when the bad token is an operator.
+    The following cases are considered:
+    1. operator instead of comma
+    1. operator instead of equal sign
+    """
     _ = current_lang.translate
     # TODO: add tests
 
     if not statement.bad_token.is_operator() or statement.prev_token == "def":
         return {}
 
-    if statement.prev_token.is_in("(,"):
-        hint = _("You cannot have operators as function arguments.\n")
-    else:
+    no_op = _("You cannot have operators as function arguments.\n")
+
+    # def test(a+b): -> def test(a, b):
+    new_statement = fixers.replace_token(
+        statement.statement_tokens, statement.bad_token, ","
+    )
+    if fixers.check_statement(new_statement):
+        hint = _("Did you mean to write a comma?\n")
+        cause = no_op + _(
+            "I suspect you made a typo and wrote `{op}` instead of a comma.\n"
+            "The following statement contains no syntax error:\n\n"
+            "    {new_statement}"
+        ).format(op=statement.bad_token, new_statement=new_statement)
+        return {"cause": cause, "suggest": hint}
+
+    # def test(a=1, b+2):  -> def test(a=1, b=2):
+    new_statement = fixers.replace_token(
+        statement.statement_tokens, statement.bad_token, "="
+    )
+    if fixers.check_statement(new_statement):
+        hint = _("Did you mean to write an equal sign?\n")
+        cause = no_op + _(
+            "I suspect you made a typo and wrote `{op}` instead of an equal sign.\n"
+            "The following statement contains no syntax error:\n\n"
+            "    {new_statement}"
+        ).format(op=statement.bad_token, new_statement=new_statement)
+        return {"cause": cause, "suggest": hint}
+
+    # def test(a,,b):  -> def test(a,b):
+    new_statement = fixers.replace_token(
+        statement.statement_tokens, statement.bad_token, ""
+    )
+    if fixers.check_statement(new_statement):
+        hint = _("Did you mean to write `{op}`?\n").format(op=statement.bad_token)
+        cause = _(
+            "I suspect you made a typo and added `{op}` by mistake.\n"
+            "The following statement contains no syntax error:\n\n"
+            "    {new_statement}"
+        ).format(op=statement.bad_token, new_statement=new_statement)
+        if statement.bad_token != ",":
+            cause = no_op + cause
+        return {"cause": cause, "suggest": hint}
+
+    # def test(a, **, b): -> def test(a, c, b):
+    if statement.prev_token.string in "(,":
+        # prevent getting an error because of repeated argument
+        unique_name = utils.unique_variable_name()
         new_statement = fixers.replace_token(
-            statement.statement_tokens, statement.bad_token, ","
+            statement.statement_tokens, statement.bad_token, unique_name
         )
         if fixers.check_statement(new_statement):
-            hint = _("Did you mean to write a comma?\n")
+            hint = _("You cannot use `{op}` as an argument.\n").format(
+                op=statement.bad_token
+            )
             cause = _(
-                "I suspect you made a typo and wrote `{op}` instead of a comma.\n"
-                "The following statement contains no syntax error:\n\n"
-                "    {new_statement}"
-            ).format(op=statement.bad_token, new_statement=new_statement)
+                "I suspect you made a typo and wrote `{op}` by mistake.\n"
+                "If you replace it by a unique variable name, the result\n"
+                "will contain no syntax error.\n"
+            ).format(op=statement.bad_token)
             return {"cause": cause, "suggest": hint}
-        hint = _("You cannot use operators with function arguments.\n")
-    cause = hint + _(
-        "You can only use identifiers (variable names) as function arguments.\n"
-    )
-    return {"cause": cause, "suggest": hint}
+
+    # The following has no corresponding unit test, but has been tested in a repl
+    if statement.prev_token.string in ("(", ","):
+        hint = _("You cannot use `{op}` as an argument.\n").format(
+            op=statement.bad_token
+        )
+        cause = hint + _(
+            "You can only use identifiers (variable names) as function arguments.\n"
+        )
+        return {"cause": cause, "suggest": hint}
+
+    return {}
 
 
 @add_statement_analyzer
 def arg_after_kwarg(statement):
+    """This is only for something with positional argument after **kwargs;
+    the case where we have a positional argument after a named argument,
+    (..., a=1, b, ...) gets a specific error message.
+    """
     _ = current_lang.translate
-    # TODO: add tests
 
     if not (
         statement.bad_token.is_identifier()
-        and statement.prev_token == statement.next_token == ","
+        and statement.prev_token == ","
+        and statement.next_token in (",", ")")
     ):
         return {}
 
     for tok in statement.tokens[0 : statement.bad_token_index]:
-        if tok == "**" or tok == "=":
+        if tok == "**":
             hint = _("Positional arguments must come before keyword arguments.\n")
             cause = hint + _(
                 "`{arg}` is a positional argument that appears after one or more\n"
