@@ -11,6 +11,7 @@ import sys
 from . import fixers
 from . import syntax_utils
 from . import statement_analyzer
+from .. import debug_helper
 from .. import utils
 from ..my_gettext import current_lang, use_www
 
@@ -67,24 +68,24 @@ def add_python_message(func):
 @add_python_message
 def assign_to_keyword(message="", statement=None):
     _ = current_lang.translate
-    if not (
-        message == "can't assign to keyword"  # Python 3.6, 3.7
-        or message == "assignment to keyword"  # Python 3.6, 3.7
-        or message == "cannot assign to keyword"  # Python 3.8
-        or message == "cannot assign to None"  # Python 3.8
-        or message == "cannot assign to True"  # Python 3.8
-        or message == "cannot assign to False"  # Python 3.8
-        or message == "cannot assign to __debug__"  # Python 3.8
-        or message == "can't assign to Ellipsis"  # Python 3.6, 3.7
-        or message == "cannot assign to Ellipsis"  # Python 3.8
-        or message == "cannot use named assignment with True"  # Python 3.8
-        or message == "cannot use named assignment with False"  # Python 3.8
-        or message == "cannot use named assignment with None"  # Python 3.8
-        or message == "cannot use named assignment with Ellipsis"  # Python 3.8
-        or message == "cannot use assignment expressions with True"  # Python 3.8
-        or message == "cannot use assignment expressions with False"  # Python 3.8
-        or message == "cannot use assignment expressions with None"  # Python 3.8
-        or message == "cannot use assignment expressions with Ellipsis"  # Python 3.8
+    if message not in (
+        "can't assign to keyword",  # Python 3.6, 3.7
+        "assignment to keyword",  # Python 3.6, 3.7
+        "cannot assign to keyword",  # Python 3.8
+        "cannot assign to None",  # Python 3.8
+        "cannot assign to True",  # Python 3.8
+        "cannot assign to False",  # Python 3.8
+        "cannot assign to __debug__",  # Python 3.8
+        "can't assign to Ellipsis",  # Python 3.6, 3.7
+        "cannot assign to Ellipsis",  # Python 3.8
+        "cannot use named assignment with True",  # Python 3.8
+        "cannot use named assignment with False",  # Python 3.8
+        "cannot use named assignment with None",  # Python 3.8
+        "cannot use named assignment with Ellipsis",  # Python 3.8
+        "cannot use assignment expressions with True",  # Python 3.8
+        "cannot use assignment expressions with False",  # Python 3.8
+        "cannot use assignment expressions with None",  # Python 3.8
+        "cannot use assignment expressions with Ellipsis",  # Python 3.8
     ):
         return {}
 
@@ -106,27 +107,28 @@ def assign_to_keyword(message="", statement=None):
 @add_python_message
 def assign_to_conditional_expression(message="", **_kwargs):
     _ = current_lang.translate
-    if (
-        message == "can't assign to conditional expression"  # Python 3.6, 3.7
-        or message == "cannot assign to conditional expression"  # Python 3.8
+    if message not in (
+        "can't assign to conditional expression",  # Python 3.6, 3.7
+        "cannot assign to conditional expression",  # Python 3.8
     ):
-        hint = _("You can only assign objects to identifiers (variable names).\n")
-        cause = _(
-            "On the left-hand side of an equal sign, you have a\n"
-            "conditional expression instead of the name of a variable.\n"
-            "A conditional expression has the following form:\n\n"
-            "    variable = object if condition else other_object"
-        )
-        return {"cause": cause, "suggest": hint}
-    return {}
+        return {}
+
+    hint = _("You can only assign objects to identifiers (variable names).\n")
+    cause = _(
+        "On the left-hand side of an equal sign, you have a\n"
+        "conditional expression instead of the name of a variable.\n"
+        "A conditional expression has the following form:\n\n"
+        "    variable = object if condition else other_object"
+    )
+    return {"cause": cause, "suggest": hint}
 
 
 @add_python_message
 def assign_to_function_call(message="", statement=None):
     _ = current_lang.translate
-    if (
-        message != "can't assign to function call"  # Python 3.6, 3.7
-        and message != "cannot assign to function call"  # Python 3.8
+    if message not in (
+        "can't assign to function call",  # Python 3.6, 3.7
+        "cannot assign to function call",  # Python 3.8
     ):
         return {}
 
@@ -164,17 +166,18 @@ def assign_to_function_call(message="", statement=None):
 @add_python_message
 def assign_to_generator_expression(message="", **_kwargs):
     _ = current_lang.translate
-    if (
-        message == "can't assign to generator expression"  # Python 3.6, 3.7
-        or message == "cannot assign to generator expression"  # Python 3.8
+    if message not in (
+        "can't assign to generator expression",  # Python 3.6, 3.7
+        "cannot assign to generator expression",  # Python 3.8
     ):
-        hint = _("You can only assign objects to identifiers (variable names).\n")
-        cause = _(
-            "On the left-hand side of an equal sign, you have a\n"
-            "generator expression instead of the name of a variable.\n"
-        )
-        return {"cause": cause, "suggest": hint}
-    return {}
+        return {}
+
+    hint = _("You can only assign objects to identifiers (variable names).\n")
+    cause = _(
+        "On the left-hand side of an equal sign, you have a\n"
+        "generator expression instead of the name of a variable.\n"
+    )
+    return {"cause": cause, "suggest": hint}
 
 
 @add_python_message
@@ -217,27 +220,24 @@ def what_kind_of_literal(literal):
 
     try:
         a = ast.literal_eval(literal)
-    except Exception:
+    except Exception:  # noqa
         return None
 
-    if isinstance(a, int):
-        return _("of type `int`")
-    elif isinstance(a, str):
-        return _("of type `str`")
-    elif isinstance(a, float):
-        return _("of type `float`")
-    elif isinstance(a, complex):
-        return _("of type `complex`")
-    elif isinstance(a, dict):
-        return _("of type `dict`")
-    elif isinstance(a, tuple):
-        return _("of type `tuple`")
-    elif isinstance(a, list):
-        return _("of type `list`")
-    elif isinstance(a, set):
-        return _("of type `set`")
-    else:
-        return None
+    kinds = (
+        (int, _("of type `int`")),
+        (complex, _("of type `complex`")),
+        (float, _("of type `float`")),
+        (str, _("of type `str`")),
+        (dict, _("of type `dict`")),
+        (list, _("of type `list`")),
+        (set, _("of type `set`")),
+        (tuple, _("of type `tuple`")),
+    )
+    for kind, result in kinds:
+        if isinstance(a, kind):
+            return result
+
+    return None
 
 
 @add_python_message
@@ -259,98 +259,121 @@ def annotated_name_cannot_be_global(message="", **_kwargs):
 @add_python_message
 def assign_to_literal(message="", statement=None):
     _ = current_lang.translate
-    if (
-        message == "can't assign to literal"  # Python 3.6, 3.7
-        or message == "cannot assign to literal"  # Python 3.8
-        or message == "cannot assign to set display"  # Python 3.8
-        or message == "cannot assign to dict display"  # Python 3.8
+    if message not in (
+        "can't assign to literal",  # Python 3.6, 3.7
+        "cannot assign to literal",  # Python 3.8
+        "cannot assign to set display",  # Python 3.8
+        "cannot assign to dict display",  # Python 3.8
     ):
-        line = statement.bad_line.rstrip()
-        info = line.split("=")
-        if len(info) == 2:
-            literal = info[0].strip()
-            name = info[1].strip()
-            if sys.version_info < (3, 8) and (
-                literal.startswith("f'{") or literal.startswith('f"{')
-            ):
-                cause = _(
-                    "You wrote an expression that has the f-string `{fstring}`\n"
-                    "on the left-hand side of the equal sign.\n"
-                    "An f-string should only appear on the right-hand "
-                    "side of an equal sign.\n"
-                ).format(fstring=statement.bad_token)
-                return {"cause": cause}
-        else:
-            literal = None
-            name = _("variable_name")
+        return {}
 
-        if len(info) == 2 and name.isidentifier():
-            # fmt: off
-            suggest = _(
-                "Perhaps you meant to write:\n\n"
-                "    {name} = {literal}\n"
-                "\n"
-            ).format(literal=literal, name=name)
-            hint = _(
-                "Perhaps you meant to write `{name} = {literal}`"
-            ).format(literal=literal, name=name)
-            # fmt: on
-        else:
-            suggest = "\n"
-            hint = _("You can only assign objects to identifiers (variable names).\n")
+    # This error can happen if we use a literal as an element of
+    # a for loop; we take care of this case first.
 
-        # Impose the right type when we know it.
-        if message == "cannot assign to set display":
-            of_type = what_kind_of_literal("{1}")
-        elif message == "cannot assign to dict display":
-            of_type = what_kind_of_literal("{1:2}")
-        else:
-            of_type = what_kind_of_literal(literal)
-        if of_type is None:
-            of_type = ""
+    tokens = statement.tokens[0 : statement.bad_token_index]
+    for_loop = False
+    for tok in tokens[::-1]:
+        if tok == "in":
+            break
+        elif tok == "for":
+            for_loop = True
+            break
 
-        if literal is None:
-            literal = "..."
-
-        cause = (
-            _(
-                "You wrote an expression like\n\n"
-                "    {literal} = {name}\n"
-                "where `{literal}`, on the left-hand side of the equal sign,\n"
-                "is or includes an actual object {of_type}\n"
-                "and is not simply the name of a variable.\n"
-            ).format(literal=literal, name=name, of_type=of_type)
-            + suggest
-        )
+    if for_loop:
+        hint = _("You can only assign objects to identifiers (variable names).\n")
+        cause = _(
+            "A for loop must have the form:\n\n"
+            "    for ... in sequence:\n\n"
+            "where `...` must contain only identifiers (variable names)\n"
+            "and not literals like `{bad_token}`.\n"
+        ).format(bad_token=statement.bad_token)
         return {"cause": cause, "suggest": hint}
-    return {}
+
+    line = statement.bad_line.rstrip()
+    info = line.split("=")
+    if len(info) == 2:
+        literal = info[0].strip()
+        name = info[1].strip()
+        if sys.version_info < (3, 8) and (
+            literal.startswith("f'{") or literal.startswith('f"{')
+        ):
+            cause = _(
+                "You wrote an expression that has the f-string `{fstring}`\n"
+                "on the left-hand side of the equal sign.\n"
+                "An f-string should only appear on the right-hand "
+                "side of an equal sign.\n"
+            ).format(fstring=statement.bad_token)
+            return {"cause": cause}
+    else:
+        literal = None
+        name = _("variable_name")
+
+    if len(info) == 2 and name.isidentifier():
+        # fmt: off
+        suggest = _(
+            "Perhaps you meant to write:\n\n"
+            "    {name} = {literal}\n"
+            "\n"
+        ).format(literal=literal, name=name)
+        hint = _(
+            "Perhaps you meant to write `{name} = {literal}`"
+        ).format(literal=literal, name=name)
+        # fmt: on
+    else:
+        suggest = "\n"
+        hint = _("You can only assign objects to identifiers (variable names).\n")
+
+    # Impose the right type when we know it.
+    if message == "cannot assign to set display":
+        of_type = what_kind_of_literal("{1}")
+    elif message == "cannot assign to dict display":
+        of_type = what_kind_of_literal("{1:2}")
+    else:
+        of_type = what_kind_of_literal(literal)
+    if of_type is None:
+        of_type = ""
+
+    if literal is None:
+        literal = "..."
+
+    cause = (
+        _(
+            "You wrote an expression like\n\n"
+            "    {literal} = {name}\n"
+            "where `{literal}`, on the left-hand side of the equal sign,\n"
+            "is or includes an actual object {of_type}\n"
+            "and is not simply the name of a variable.\n"
+        ).format(literal=literal, name=name, of_type=of_type)
+        + suggest
+    )
+    return {"cause": cause, "suggest": hint}
 
 
 @add_python_message
 def assign_to_operator(message="", statement=None):
     _ = current_lang.translate
     line = statement.bad_line.rstrip()
-    if (
-        message == "can't assign to operator"  # Python 3.6, 3.7
-        or message == "cannot assign to operator"  # Python 3.8
+    if message not in (
+        "can't assign to operator",  # Python 3.6, 3.7
+        "cannot assign to operator",  # Python 3.8
     ):
-        cause = _(
-            "You wrote an expression that includes some mathematical operations\n"
-            "on the left-hand side of the equal sign which should be\n"
-            "only used to assign a value to a variable.\n"
-        )
-        name = could_be_identifier(line)
-        if name:
-            hint = _("Did you mean `{name}`?\n").format(name=name)
-            cause += _(
-                "Perhaps you meant to write `{name}` instead of `{original}`\n"
-            ).format(name=name, original=name.replace("_", "-"))
-            return {"cause": cause, "suggest": hint}
-        else:
-            hint = _("You can only assign objects to identifiers (variable names).\n")
-            return {"cause": cause, "suggest": hint}
+        return {}
 
-    return {}
+    cause = _(
+        "You wrote an expression that includes some mathematical operations\n"
+        "on the left-hand side of the equal sign which should be\n"
+        "only used to assign a value to a variable.\n"
+    )
+    name = could_be_identifier(line)
+    if name:
+        hint = _("Did you mean `{name}`?\n").format(name=name)
+        cause += _(
+            "Perhaps you meant to write `{name}` instead of `{original}`\n"
+        ).format(name=name, original=name.replace("_", "-"))
+        return {"cause": cause, "suggest": hint}
+
+    hint = _("You can only assign objects to identifiers (variable names).\n")
+    return {"cause": cause, "suggest": hint}
 
 
 def could_be_identifier(line):
@@ -362,11 +385,27 @@ def could_be_identifier(line):
                 if lhs.isidentifier():
                     return lhs
         return ""
-    except Exception:
+    except Exception:  # noqa
         print("exception raised")
         return ""
 
 
+@add_python_message
+def augmented_assignment_with_literal(message="", statement=None):
+    _ = current_lang.translate
+    if message != "cannot use assignment expressions with literal":
+        return {}
+
+    hint = _("You can only assign objects to identifiers (variable names).\n")
+
+    cause = _(
+        "You cannot use the augmented assignment operator `:=`,\n"
+        "sometimes called the walrus operator, with literals like `{bad_token}`.\n"
+        "You can only assign objects to identifiers (variable names).\n"
+    ).format(bad_token=statement.bad_token)
+    return {"cause": cause, "suggest": hint}
+
+
 @add_python_message
 def both_nonlocal_and_global(message="", statement=None):
     _ = current_lang.translate
@@ -407,36 +446,37 @@ def continue_outside_loop(message="", **_kwargs):
 @add_python_message
 def delete_function_call(message="", statement=None):
     _ = current_lang.translate
-    if (
-        message == "can't delete function call"  # Python 3.6, 3.7
-        or message == "cannot delete function call"  # Python 3.8
+    if message not in (
+        "can't delete function call",  # Python 3.6, 3.7
+        "cannot delete function call",  # Python 3.8
     ):
-        line = statement.bad_line.rstrip()
-        correct = "del {name}".format(name=statement.bad_token)
-        cause = _(
-            "You attempted to delete a function call\n\n"
-            "    {line}\n"
-            "instead of deleting the function's name\n\n"
-            "    {correct}\n"
-        ).format(line=line, correct=correct)
-        return {"cause": cause}
-    return {}
+        return {}
+
+    line = statement.bad_line.rstrip()
+    correct = "del {name}".format(name=statement.bad_token)
+    cause = _(
+        "You attempted to delete a function call\n\n"
+        "    {line}\n"
+        "instead of deleting the function's name\n\n"
+        "    {correct}\n"
+    ).format(line=line, correct=correct)
+    return {"cause": cause}
 
 
 @add_python_message
 def delete_x(message="", statement=None):
     _ = current_lang.translate
-    if not (
-        message == "can't delete keyword"  # Python 3.6, 3.7
-        or message == "can't delete literal"
-        or message == "cannot delete literal"  # Python 3.8
-        or message == "cannot delete None"
-        or message == "cannot delete True"
-        or message == "cannot delete False"
+    if message not in (
+        "can't delete keyword",  # Python 3.6, 3.7
+        "can't delete literal",
+        "cannot delete literal",
+        "cannot delete None",
+        "cannot delete True",
+        "cannot delete False",
     ):
         return {}
 
-    if statement.bad_token.is_in(["None", "True", "False"]):
+    if statement.bad_token.string in ("None", "True", "False"):
         cause = _("You cannot delete the constant `{constant}`.\n").format(
             constant=statement.bad_token
         )
@@ -473,9 +513,10 @@ def eol_while_scanning_string_literal(message="", statement=None):
     ):
         hint = _("Did you forget a closing quote?\n")
         cause = _(
-            "You starting writing a string with a single or double quote\n"
+            "You started writing a string with a single or double quote\n"
             "but never ended the string with another quote on that line.\n"
         )
+        # skipcq: PYL-R1714
         # second if case for Python 3.10
         if statement.prev_token == "\\" or statement.bad_line[-2] == "\\":
             cause += _(
@@ -777,7 +818,7 @@ def no_binding_for_nonlocal(message="", **_kwargs):
 
 
 @add_python_message
-def unexpected_character_after_continuation(message="", **_kwargs):
+def unexpected_character_after_continuation(message="", statement=None):
     _ = current_lang.translate
     if "unexpected character after line continuation character" not in message:
         return {}
@@ -785,9 +826,34 @@ def unexpected_character_after_continuation(message="", **_kwargs):
     cause = _(
         "You are using the continuation character `\\` outside of a string,\n"
         "and it is followed by some other character(s).\n"
-        "I am guessing that you forgot to enclose some content in a string.\n"
-        "\n"
     )
+
+    # For 3.9.0 and 3.10.0a6 (and possibly others), the new peg parser is
+    # not giving us the correct location for the error; so we need to find it.
+    bad_token = statement.bad_token
+    if statement.prev_token != "\\":
+        found_continuation = False
+        for tok in statement.tokens:
+            if tok == "\\":
+                found_continuation = True
+                continue
+
+            if found_continuation:
+                bad_token = tok
+                break
+        else:
+            debug_helper.log("Could not find bad token after continuation character.")
+
+    if bad_token.is_number():
+        number = bad_token
+        cause += _(
+            "I am guessing that you wanted to divide by the number {number} \n"
+            "and wrote \\ instead of /."
+        ).format(number=number)
+        hint = _("Did you mean to divide by {number}?\n").format(number=number)
+        return {"cause": cause, "suggest": hint}
+
+    cause += _("I am guessing that you forgot to enclose some content in a string.\n")
     return {"cause": cause}
 
 
@@ -888,7 +954,7 @@ def python2_print(message="", **_kwargs):
 @add_python_message
 def cannot_use_starred_expression(message="", **_kwargs):
     _ = current_lang.translate
-    if not message == "can't use starred expression here":
+    if message != "can't use starred expression here":
         return {}
 
     cause = _(
@@ -903,7 +969,7 @@ def cannot_use_starred_expression(message="", **_kwargs):
 @add_python_message
 def return_outside_function(message="", **_kwargs):
     _ = current_lang.translate
-    if not message == "'return' outside function":
+    if message != "'return' outside function":
         return {}
 
     cause = _("You can only use a `return` statement inside a function or method.\n")
@@ -913,7 +979,7 @@ def return_outside_function(message="", **_kwargs):
 @add_python_message
 def too_many_nested_blocks(message="", **_kwargs):
     _ = current_lang.translate
-    if not message == "too many statically nested blocks":
+    if message != "too many statically nested blocks":
         return {}
 
     hint = _("Seriously?\n")
@@ -928,7 +994,7 @@ def too_many_nested_blocks(message="", **_kwargs):
 @add_python_message
 def named_arguments_must_follow_bare_star(message="", **_kwargs):
     _ = current_lang.translate
-    if not message == "named arguments must follow bare *":
+    if message != "named arguments must follow bare *":
         return {}
 
     hint = _("Did you forget something after `*`?\n")
@@ -943,7 +1009,7 @@ def named_arguments_must_follow_bare_star(message="", **_kwargs):
 @add_python_message
 def you_found_it(message="", **_kwargs):
     _ = current_lang.translate
-    if not message == "You found it!":
+    if message != "You found it!":
         return {}
 
     cause = _(
@@ -975,21 +1041,20 @@ def from__future__not_defined(message="", **_kwargs):
         cause += "\n" + available
         return {"cause": cause}
 
-    else:
-        names = __future__.all_feature_names
-        similar = utils.get_similar_words(feature, names)
-        if similar:
-            hint = _("Did you mean `{name}`?\n").format(name=similar[0])
-            cause = _(
-                "Instead of `{feature}`, perhaps you meant to import `{name}`.\n"
-            ).format(feature=feature, name=similar[0])
-            return {"cause": cause, "suggest": hint}
-        else:
-            cause = _(
-                "`{feature}` is not a valid feature of module `__future__`.\n"
-            ).format(feature=feature)
-            cause += "\n" + available
-            return {"cause": cause}
+    names = __future__.all_feature_names
+    similar = utils.get_similar_words(feature, names)
+    if similar:
+        hint = _("Did you mean `{name}`?\n").format(name=similar[0])
+        cause = _(
+            "Instead of `{feature}`, perhaps you meant to import `{name}`.\n"
+        ).format(feature=feature, name=similar[0])
+        return {"cause": cause, "suggest": hint}
+
+    cause = _("`{feature}` is not a valid feature of module `__future__`.\n").format(
+        feature=feature
+    )
+    cause += "\n" + available
+    return {"cause": cause}
 
 
 @add_python_message
@@ -1053,7 +1118,7 @@ def proper_decimal_or_octal_number(prev_str, bad_str):
     if not (set(prev_str).issubset("_0") and prev_str.startswith("0")):  # noqa
         return {}
 
-    if prev_str == "0" and set(bad_str).issubset("01234567_"):
+    if prev_str == "0" and set(bad_str).issubset("01234567_"):  # noqa
         correct = "0o" + bad_str
         hint = _("Did you mean `{num}`?\n").format(num=correct)
         cause = _(
@@ -1062,7 +1127,8 @@ def proper_decimal_or_octal_number(prev_str, bad_str):
             "a decimal integer and did not know that it could not start with zeros.\n"
         ).format(num=correct)
         return {"cause": cause, "suggest": hint}
-    elif set(bad_str).issubset("0123456789_"):
+
+    if set(bad_str).issubset("0123456789_"):
         correct = bad_str.lstrip("_")
         hint = _("Did you mean `{num}`?\n").format(num=correct)
         cause = _(
@@ -1078,7 +1144,7 @@ def proper_decimal_or_octal_number(prev_str, bad_str):
 def invalid_token(message="", statement=None):
     # Seen this for Python 3.6, 3.7 for would-be decimal number starting with zero.
     _ = current_lang.translate
-    if not (message == "invalid token"):
+    if message != "invalid token":
         return {}
 
     prev_str = statement.prev_token.string
@@ -1106,7 +1172,7 @@ def leading_zeros_in_decimal_integers(message="", statement=None):
 def forgot_paren_around_comprehension(message="", **_kwargs):
     # Python 3.10+
     _ = current_lang.translate
-    if not message == "did you forget parentheses around the comprehension target?":
+    if message != "did you forget parentheses around the comprehension target?":
         return {}
 
     # message same as from statement_analyzer.comprehension_condition_or_tuple
@@ -1200,3 +1266,21 @@ def bracket_was_expected(message="", statement=None):
     if rephrased_cause:
         cause = rephrased_cause["cause"]
     return {"cause": cause, "suggest": hint}
+
+
+@add_python_message
+def invalid_double_star_operator(message="", **_kwargs):
+    _ = current_lang.translate
+
+    # Used to be "invalid syntax" prior to Python version 3.10
+    if (
+        message == "f-string: can't use double starred expression here"  # 3.10.0a7
+        or message == "f-string: cannot use double starred expression here"  # future?
+    ):
+        cause = _(
+            "The double star operator `**` is likely interpreted to mean that\n"
+            "dict unpacking is to be used which is not allowed or does not make sense here.\n"
+        )
+        return {"cause": cause}
+
+    return {}
